#include "stdafx.h"

#include "FieldPatch.h"

// FieldPatch

CreateFrameOperation<FieldPatch> g_createFieldPatch("FieldPatch");

FieldPatch::FieldPatch(CVclipDoc& owner, 
	LONG aSourceFrame, LONG aSourceOrder, int aSourceX, int aSourceY, int aWidth, int aHeight,
	LONG aTargetFrame, LONG aTargetOrder, int aTargetX, int aTargetY, HOverlay mode)
: PatchOperation(owner, aTargetFrame, aTargetX, aTargetY, aWidth, aHeight),
	m_sourceFrame(aSourceFrame), m_sourceOrder(aSourceOrder), m_targetOrder(aTargetOrder),
	m_sourceOrigin(aSourceX, aSourceY), 
	m_mode(mode)
{
}

LONG FieldPatch::sourceFrame() const
{
	return m_sourceFrame;
}
LONG FieldPatch::sourceOrder() const
{
	return m_sourceOrder;
}
LONG FieldPatch::targetOrder() const
{
	return m_targetOrder;
}

CPoint FieldPatch::sourceOrigin() const
{
	return m_sourceOrigin;
}
CRect FieldPatch::sourceRect() const
{
	return CRect(m_sourceOrigin, size());
}
int FieldPatch::sourceX() const
{
	return m_sourceOrigin.x;
}
int FieldPatch::sourceY() const
{
	return m_sourceOrigin.y;
}

HOverlay FieldPatch::mode() const
{
	return m_mode;
}
void FieldPatch::setMode(HOverlay mode)
{
	m_mode = mode;
	onModified();
}

bool FieldPatch::usesFrame(LONG frame) const
{
	return m_sourceFrame == frame;
}

const char* FieldPatch::typeCode() const
{
	return "FieldPatch";
}

void FieldPatch::writeTo(std::ostream& stream) const
{
	stream             <<
		m_sourceFrame    << " " <<
		m_sourceOrder    << " " <<
		m_sourceOrigin.x << " " <<
		m_sourceOrigin.y << " " <<
		width()          << " " <<
		height()         << " " <<
		targetFrame()    << " " <<
		m_targetOrder    << " " <<
		targetX()        << " " <<
		targetY()        << " " <<
		m_mode;
}

void FieldPatch::readFrom(std::istream& stream)
{
	int width, height, targetX, targetY;
	LONG targetFrame;
	stream             >>
		m_sourceFrame    >>
		m_sourceOrder    >>
		m_sourceOrigin.x >>
		m_sourceOrigin.y >>
		width            >>
		height           >>
		targetFrame      >>
		m_targetOrder    >>
		targetX          >>
		targetY          >>
		m_mode;
	setTargetFrame(targetFrame);
	moveTo(CPoint(targetX, targetY));
	resize(CSize(width, height));
}

void FieldPatch::accept(IOperationVisitor& visitor)
{
	return visitor(*this);
}

