#include "stdafx.h"

#include "FramePatch.h"


// FramePatch

CreateFrameOperation<FramePatch> g_createFramePatch("FramePatch");

FramePatch::FramePatch(CVclipDoc& owner, 
	LONG aSourceFrame, int aSourceX, int aSourceY, int aWidth, int aHeight,
	LONG aTargetFrame, int aTargetX, int aTargetY, HOverlay mode)
: PatchOperation(owner, aTargetFrame, aTargetX, aTargetY, aWidth, aHeight),
	m_sourceFrame(aSourceFrame), 
	m_sourceOrigin(aSourceX, aSourceY), 
	m_mode(mode)
{
}

LONG FramePatch::sourceFrame() const
{
	return m_sourceFrame;
}

CPoint FramePatch::sourceOrigin() const
{
	return m_sourceOrigin;
}
CRect FramePatch::sourceRect() const
{
	return CRect(m_sourceOrigin, size());
}
int FramePatch::sourceX() const
{
	return m_sourceOrigin.x;
}
int FramePatch::sourceY() const
{
	return m_sourceOrigin.y;
}

HOverlay FramePatch::mode() const
{
	return m_mode;
}

void FramePatch::setMode(HOverlay mode)
{
	m_mode = mode;
	onModified();
}

bool FramePatch::usesFrame(LONG frame) const
{
	return m_sourceFrame == frame;
}

const char* FramePatch::typeCode() const
{
	return "FramePatch";
}

void FramePatch::writeTo(std::ostream& stream) const
{
	stream             <<
		m_sourceFrame    << " " <<
		m_sourceOrigin.x << " " <<
		m_sourceOrigin.y << " " <<
		width()          << " " <<
		height()         << " " <<
		targetFrame()    << " " <<
		targetX()        << " " <<
		targetY()        << " " <<
		m_mode;
}

void FramePatch::readFrom(std::istream& stream)
{
	int width, height, targetX, targetY;
	LONG targetFrame;
	stream              >>
		m_sourceFrame     >>
		m_sourceOrigin.x  >>
		m_sourceOrigin.y  >>
		width             >>
		height            >>
		targetFrame       >>
		targetX           >>
		targetY           >>
		m_mode;
	setTargetFrame(targetFrame);
	moveTo(CPoint(targetX, targetY));
	resize(CSize(width, height));
}

void FramePatch::accept(IOperationVisitor& visitor)
{
	return visitor(*this);
}


