// vclipDoc.h : interface of the CVclipDoc class
//
#pragma once

#include "FrameOperation.h"

#include <iostream>
#include <istream>
#include <ostream>
#include <string>
#include <vector>
#include <map>
#include <functional>

class CVclipDoc : public CDocument
{
protected: // create from serialization only
	CVclipDoc();
	DECLARE_DYNCREATE(CVclipDoc)

// Attributes
private:
	std::string m_source;

	typedef std::vector<PFrameOperation> Operations;
	Operations m_operations;

public:
	typedef Operations::iterator iterator;
	typedef Operations::const_iterator const_iterator;
	
	template <typename IteratorT>
	class basic_range : public std::pair<IteratorT, IteratorT>
	{
	public:
		basic_range(const std::pair<IteratorT, IteratorT>& other)
			: std::pair<IteratorT, IteratorT>(other) {}

		template <typename FunctorT>
		FunctorT for_each(const FunctorT& functor) const { return std::for_each(first, second, functor); }

		template <typename FunctorT>
		FunctorT for_each_except(PFrameOperation op, FunctorT functor) const
		{
			for (IteratorT it = first; it != second; ++it) if (op != *it) functor(*it);
			return functor;
		}

		bool empty() const { return first == second; }
	};
	typedef basic_range<const_iterator> const_range;
	typedef basic_range<iterator> range;

	const_range operations(LONG frame) const;
	range operations(LONG frame);
	const_iterator begin() const;
	const_iterator end() const;
	bool needFrame(LONG frame) const;

	const std::string& source() const;

// Operations
public:

	void setSource(const std::string& source);

	PFrameOperation add(const PFrameOperation& op);
	void erase(const PFrameOperation& op);

	void compile(LPCTSTR lpszPathName, bool evenCoords, int totalFrames, const CRect& frameSize);

	void sendToBack(PFrameOperation op);
	bool canSendFurther(PFrameOperation op) const;
	void sendFurther(PFrameOperation op);
	bool canBringNearer(PFrameOperation op) const;
	void bringNearer(PFrameOperation op);
	void bringToFront(PFrameOperation op);

// Overrides
public:
	virtual BOOL OnNewDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	virtual void Serialize(CArchive& ar);

// Implementation
public:
	virtual ~CVclipDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()
};

class Compiler : public std::unary_function<PFrameOperation, void>, 
	public IOperationVisitor
{
private:
	std::ostream* m_stream;
	int m_frame;
	bool m_evenCoords, m_needPlus, m_needCR;
	CRect m_frameSize;

	int roundDown(int n) const;
	int roundUp(int n) const;

public:
	explicit Compiler(std::ostream& stream, bool evenCoords, const CRect& frameSize);
	void source(const std::string& source);
	void operator()(const PFrameOperation& op);
	void operator()(FramePatch& patch);
	void operator()(FieldPatch& patch);
	void operator()(FrameImage& image);
	void operator()(FieldRebuild& rebuild);
	void operator()(MotionInterpolate& inter);
	void empty();
	void end(int totalFrames);
};

